@@ -26,12 +26,6 @@ def parse_options():
         help="run interactively after processing the given files"
     )
 
-    add_option(
-        "-d", "",
-        action="store_true", default=False, dest="debug",
-        help="enable debugging output"
-    )
-
     add_option(
         "-S", "",
         action="store_true", default=False, dest="nosys",
