@@ -6,28 +6,25 @@ from utils import method
 
 class Message(Object):
 
-    def __init__(self, name, *args, **kwargs):
-        value = kwargs.get("value")
+    def __init__(self, name=None, value=None, args=None):
         super(Message, self).__init__(value=value)
 
         self.name = name
+        self.value = value
+        self.args = args if args is not None else []
 
-        self._args = list(args)
-        for arg in args:
-            arg.previous = self
-
+        self.call = args is not None
         self.terminator = self.value is None and name in ["\r", "\n", ";"]
 
-        self._next = None
         self._previous = self
-
         self._first = self
         self._last = self
+        self._next = None
 
         self.create_methods()
-        self.parent = runtime.state.find("Object")
+        self.parent = runtime.find("Object")
 
-    def __str__(self):
+    def __repr__(self):
         messages = []
 
         next = self
@@ -41,16 +38,11 @@ class Message(Object):
 
         return " ".join(messages)
 
-    __repr__ = __str__
-
     def eval(self, receiver, context=None, m=None, target=None):
         context = receiver if context is None else context
         m = self if m is None else m
 
         while m is not None:
-            if runtime.state.opts and runtime.state.opts.debug:
-                print "eval {0:s} in {1:s} at {2:s}".format(repr(m.name), repr(context), repr(receiver))
-
             if m.terminator:
                 value = context
             elif m.value is not None:
@@ -71,13 +63,18 @@ class Message(Object):
 
         try:
             returnValue = runtime.state.value if runtime.state.value is not None else receiver
-            if runtime.state.opts and runtime.state.opts.debug:
-                print "return {0:s}".format(repr(returnValue))
-            context["_"] = returnValue
+            runtime.find("Root")["_"] = returnValue
             return returnValue
         finally:
             runtime.state.value = None
 
+    @method()
+    def init(self, receiver, context, m):
+        receiver._previous = receiver
+        receiver._first = receiver
+        receiver._last = receiver
+        receiver._next = None
+
     @method("eval")
     def _eval(self, receiver, context, m, target=None):
         target = target.eval(context) if target is not None else context
@@ -93,12 +90,12 @@ class Message(Object):
 
     @method("args")
     def getArgs(self, receiver, context, m):
-        return runtime.find("List").clone(self.args)
+        return runtime.find("List").clone(receiver.args)
 
     @method()
-    def argsEvaluatedIn(self, receiver, context, m, target=None):
+    def evalArgs(self, receiver, context, m, target=None):
         target = target.eval(context) if target is not None else context
-        args = [arg.eval(target) for arg in self.args]
+        args = [arg.eval(target) for arg in receiver.args]
         return runtime.find("List").clone(list(args))
 
     @method("first")
@@ -123,7 +120,8 @@ class Message(Object):
 
     @method()
     def setArgs(self, receiver, context, m, *args):
-        receiver.args = tuple(args)
+        receiver.args = list(args)
+        receiver.call = True
         return receiver
 
     @method()
@@ -133,11 +131,7 @@ class Message(Object):
 
     @method()
     def setNext(self, receiver, context, m, message):
-        result = message.eval(context)
-        if isinstance(result, Message):
-            receiver.next = result
-        else:
-            receiver.next = message
+        receiver.next = message.eval(context)
         return receiver
 
     @property
@@ -160,12 +154,15 @@ class Message(Object):
 
     @next.setter
     def next(self, message):
-        if self._first is None:
-            self._first = self
         message._first = self._first
         message._previous = self
+
         self._next = message
-        self._last = message
+
+        next = self._first
+        while next is not None:
+            next._last = message
+            next = next._next
 
     @property
     def last(self):
