@@ -1,7 +1,9 @@
 import sys
 from copy import copy
+from collections import OrderedDict
 from inspect import getmembers, ismethod
 
+
 import runtime
 from .states import NormalState
 from .errors import AttributeError, TypeError
@@ -11,7 +13,7 @@ from .states import BreakState, ContinueState, ReturnState
 
 class Object(object):
 
-    __slots__ = ("attrs", "parent", "_state", "value", "traits", "behaviors",)
+    __slots__ = ("attrs", "binding", "parent", "state", "value", "traits", "behaviors",)
 
     def __init__(self, value=Null, methods=False):
         super(Object, self).__init__()
@@ -23,7 +25,8 @@ class Object(object):
         self.traits = []
         self.behaviors = {}
 
-        self._state = NormalState()
+        self.binding = None
+        self.state = NormalState()
 
         if methods:
             self.create_methods()
@@ -68,19 +71,17 @@ class Object(object):
                     return parent[key]
                 parent = parent.parent
 
-        if hasattr(self, "forward"):
-            try:
-                return self.forward(key)
-            except:
-                raise AttributeError("%s has no attribute %r" % (self, key))
-        raise AttributeError("%s has no attribute %r" % (self, key))
+        try:
+            return self.forward(key)
+        except:
+            raise AttributeError("{0:s} has no attribute {1:s}".format(self.type, repr(key)))
 
     def __setitem__(self, key, value):
         self.attrs[key] = value
 
     def __addtrait__(self, trait):
         attrs = ((k, v) for k, v in trait.attrs.items())
-        methods = ((k, v) for k, v in attrs if v.type == "Method")
+        methods = ((k, v) for k, v in attrs if v.type == "Block")
         behaviors = ((k, v) for k, v in methods if not k in self.attrs)
 
         self.behaviors.update(behaviors)
@@ -89,35 +90,29 @@ class Object(object):
     def __deltrait__(self, trait):
         for k, v in trait.attrs.items():
             del self.behaviors[k]
-        self.traits.remvoe(trait)
+        self.traits.remove(trait)
 
-    def __str__(self):
-        default = "%s_%s" % (self.type, hex(id(self)))
-        return str(self.value) if self.value is not Null else default
-
-    __repr__ = __str__
+    def __repr__(self):
+        type = "{0:s}({1:s})".format(self.binding, self.type) if self.binding is not None else self.type
+        default = "{0:s} at {1:s}".format(type, hex(id(self)))
+        return repr(self.value) if self.value is not Null else default
 
     def clone(self, value=Null):
         obj = copy(self)
 
         obj.attrs = {}
         obj.parent = self
+        obj.value = value if value is not Null else obj.value
 
-        if value is not Null:
-            obj.value = value
+        obj.traits = []
+        obj.behaviors = {}
+
+        obj.state = NormalState()
 
         return obj
 
     def forward(self, key):
-        return runtime.state.find(key)
-
-    @property
-    def state(self):
-        return self._state
-
-    @state.setter
-    def state(self, state):
-        self._state = state
+        return runtime.find(key)
 
     @property
     def type(self):
@@ -129,14 +124,14 @@ class Object(object):
     def _del(self, receiver, context, m, key):
         key = key.eval(context)
         del receiver[key]
-        return runtime.state.find("None")
+        return runtime.find("None")
 
     @method()
     def has(self, receiver, context, m, key):
         key = key.eval(context)
         if key in receiver:
-            return runtime.state.find("True")
-        return runtime.state.find("False")
+            return runtime.find("True")
+        return runtime.find("False")
 
     @method()
     def set(self, receiver, context, m, key, value):
@@ -194,9 +189,9 @@ class Object(object):
 
         # Evaluate kwargs first
         ctx = runtime.find("Object").clone()
-        kwargs = dict([(arg.name, arg.eval(ctx, context)) for arg in args if arg.name == "set"])
+        kwargs = OrderedDict([(arg.args[0].name, arg.eval(ctx)) for arg in args if arg.name == "set"])
 
-        args = [arg.name for arg in args if not arg.name == "set"]
+        args = [arg for arg in args if not arg.name == "set"]
 
         from .block import Block
         return Block(body, args, kwargs, context)
@@ -207,9 +202,9 @@ class Object(object):
 
         # Evaluate kwargs first
         ctx = runtime.find("Object").clone()
-        kwargs = dict([(arg.name, arg.eval(ctx, context)) for arg in args if arg.name == "set"])
+        kwargs = OrderedDict([(arg.args[0].name, arg.eval(ctx)) for arg in args if arg.name == "set"])
 
-        args = [arg.name for arg in args if not arg.name == "set"]
+        args = [arg for arg in args if not arg.name == "set"]
 
         from .block import Block
         return Block(body, args, kwargs)
@@ -223,10 +218,6 @@ class Object(object):
             vars, expression = args[:-1], args[-1]
 
             for item in receiver:
-                if context.state.isContinue:
-                    context.state = NormalState()
-                    continue
-
                 if len(vars) == 2:
                     context[vars[0].name], context[vars[1].name] = item
                 elif len(vars) == 1:
@@ -235,10 +226,15 @@ class Object(object):
                 result = expression.eval(context)
 
                 if context.state.stop:
-                    return context.state.returnValue
+                    if context.state.isContinue:
+                        context.state = NormalState()
+                        continue
+                    else:
+                        return context.state.returnValue
             return result
         finally:
-            context.state = NormalState()
+            if not context.state.isReturn:
+                context.state = NormalState()
 
     @method("while")
     def _while(self, receiver, context, m, condition, expression):
@@ -246,17 +242,18 @@ class Object(object):
             result = runtime.find("None")
 
             while condition.eval(context):
-                if context.state.isContinue:
-                    context.state = NormalState()
-                    continue
-
                 result = expression.eval(context)
 
                 if context.state.stop:
-                    return context.state.returnValue
+                    if context.state.isContinue:
+                        context.state = NormalState()
+                        continue
+                    else:
+                        return context.state.returnValue
             return result
         finally:
-            context.state = NormalState()
+            if not context.state.isReturn:
+                context.state = NormalState()
 
     @method("continue")
     def _continue(self, receiver, context, m):
@@ -274,50 +271,11 @@ class Object(object):
         context.state = ReturnState(value)
         return receiver
 
-    # I/O
-
-    @method("print")
-    def _print(self, receiver, context, m):
-        sys.stdout.write("%s" % receiver.value)
-        return receiver
-
-    @method()
-    def println(self, receiver, context, m):
-        sys.stdout.write("%s\n" % receiver.value)
-        return receiver
-
-    @method()
-    def write(self, receiver, context, m, *args):
-        args = [arg.eval(context) for arg in args]
-        sys.stdout.write("".join([str(arg) for arg in args]))
-        return runtime.find("None")
-
-    @method()
-    def writeln(self, receiver, context, m, *args):
-        args = [arg.eval(context) for arg in args]
-        sys.stdout.write("".join([str(arg) for arg in args]))
-        sys.stdout.write("\n")
-        return runtime.find("None")
-
     # Introspection
 
-    @method("state")
-    def __state(self, receiver, context, m):
-        TrueValue = runtime.find("True")
-        FalseValue = runtime.find("False")
-
-        obj = self.clone()
-
-        obj["type"] = runtime.find("String").clone(self.state.__class__.__name__)
-
-        obj["isContinue"] = TrueValue if self.state.isContinue else FalseValue
-        obj["isReturn"] = TrueValue if self.state.isReturn else FalseValue
-        obj["isNormal"] = TrueValue if self.state.isNormal else FalseValue
-        obj["isBreak"] = TrueValue if self.state.isBreak else FalseValue
-
-        obj["returnValue"] = self.state.returnValue
-
-        return obj
+    @method("type")
+    def getType(self, receiver, context, m):
+        return runtime.find("String").clone(receiver.type)
 
     @method("parent")
     def _parent(self, receiver, context, m):
@@ -325,10 +283,6 @@ class Object(object):
             return receiver.parent
         return receiver
 
-    @method("type")
-    def _type(self, receiver, context, m):
-        return runtime.find("String").clone(receiver.type)
-
     @method("value")
     def _value(self, receiver, context, m):
         if receiver.value is not Null:
@@ -356,6 +310,14 @@ class Object(object):
 
     # Object Operations
 
+    @method(":")
+    def primitive(self, receiver, context, m, method, *args):
+        method = method.name
+        args = [arg.eval(context).value for arg in args]
+        if hasattr(receiver, method):
+            return runtime.state.tomio(getattr(receiver, method)(*args))
+        raise AttributeError("{0:s} has no attribute {1:s}".format(receiver.type, repr(method)))
+
     @method()
     def evalArg(self, receiver, context, m, arg):
         return arg.eval(receiver, context)
@@ -365,19 +327,18 @@ class Object(object):
         expression.eval(receiver)
         return receiver
 
-    @method()
-    def super(self, receiver, context, m):
-        return receiver["self"].parent
-
     @method("clone")
     def _clone(self, receiver, context, m, *args):
         object = receiver.clone()
 
+        if m.first.previous.name == "set":
+            object.binding = m.first.previous.args[0].name
+
         try:
             m = runtime.find("Message").clone()
             m.name = "init"
             m.args = args
-            m.eval(object, context)
+            m.eval(object, context, m)
         except AttributeError:
             pass
 
@@ -404,20 +365,6 @@ class Object(object):
     def cmp(self, receiver, context, m, other):
         return runtime.find("Number").clone(cmp(receiver, other.eval(context)))
 
-    @method("and")
-    def _and(self, receiver, context, m, other):
-        return self.clone(receiver and other.eval(context))
-
-    @method("or")
-    def _or(self, receiver, context, m, other):
-        return self.clone(receiver or other.eval(context))
-
-    @method("not")
-    def _not(self, receiver, context, m, value=None):
-        value = value.eval(context) if value is not None else receiver
-        test = not bool(value)
-        return runtime.find("True") if test else runtime.find("False")
-
     # Type Conversion
 
     @method("bool")
