@@ -13,7 +13,7 @@ root = None
 state = None
 
 
-def init(args=None, opts=None, reinit=False):
+def init(args=[], opts=None):
     global root, state
 
     from state import State
@@ -24,11 +24,12 @@ def init(args=None, opts=None, reinit=False):
     state.create_objects()
 
     if opts is None or (opts is not None and not opts.nosys):
-        for resource in resource_listdir(__package__, "lib"):
-            filename = resource_filename(__package__, path.join("lib", resource))
+        for resource in resource_listdir(__package__, path.join("lib", "bootstrap")):
+            filename = resource_filename(__package__, path.join("lib", "bootstrap", resource))
             state.load(filename)
 
 
 def find(name):
     global root
+
     return root[name]
